@@ -30,116 +30,194 @@
 //! compress, hash and displace algorithm to use as little memory as possible while still performing
 //! fast lookups.
 //!
+//! It also comes with a B+ tree implementation that is built for maximum performance by splitting
+//! data into easily-pageable 4KB segments. This makes it perfect for building immutable databases
+//! and structures for bulk data.
+//!
 //! rkyv also has support for contextual serialization, deserialization, and validation. It can
 //! properly serialize and deserialize shared pointers like `Rc` and `Arc`, and can be extended to
 //! support custom contextual types.
 //!
-//! One of the most impactful features made possible by rkyv is the ability to serialize trait
-//! objects and use them *as trait objects* without deserialization. See the `archive_dyn` crate for
-//! more details.
+//! Finally, rkyv makes it possible to serialize trait objects and use them *as trait objects*
+//! without deserialization. See the `archive_dyn` crate for more details.
 //!
 //! ## Tradeoffs
 //!
-//! rkyv is designed primarily for loading bulk game data as efficiently as possible. While rkyv is
-//! a great format for final data, it lacks a full schema system and isn't well equipped for data
-//! migration. Using a serialization library like serde can help fill these gaps, and you can use
-//! serde with the same types as rkyv conflict-free.
+//! While rkyv is a great format for final data, it lacks a full schema system and isn't well
+//! equipped for data migration and schema upgrades. If your use case requires these capabilities,
+//! you may need additional libraries the build these features on top of rkyv. You can use other
+//! serialization frameworks like serde with the same types as rkyv conflict-free.
 //!
 //! ## Features
 //!
-//! - `const_generics`: Improves the trait implementations for arrays with support for all lengths
-//!   (enabled by default)
-//! - `size_64`: Archives `*size` as `*64` instead of `*32`. This is for large archive support
-//! - `specialization`: Enables support for the unstable specialization feature for increased
-//!   performance for a few specific cases
-//! - `std`: Enables standard library support (enabled by default)
+//! - `alloc`: Enables types that require the `alloc` crate. Enabled by default.
+//! - `arbitrary_enum_discriminant`: Enables the `arbitrary_enum_discriminant` feature for stable
+//!   stable multibyte enum discriminants using `archive_le` and `archive_be`. Requires nightly.
+//! - `archive_be`: Forces archives into a big-endian format. This guarantees cross-endian
+//!   compatibility optimized for big-endian architectures.
+//! - `archive_le`: Forces archives into a little-endian format. This guarantees cross-endian
+//!   compatibility optimized for little-endian architectures.
+//! - `copy`: Enables copy optimizations for packed copyable data types. Requires nightly.
+//! - `copy_unsafe`: Automatically opts all potentially copyable types into copy optimization. This
+//!   broadly improves performance but may cause uninitialized bytes to be copied to the output.
+//!   Requires nightly.
+//! - `size_16`: Archives integral `*size` types as 16-bit integers. This is intended to be used
+//!   only for small archives and may not handle large, more general data.
+//! - `size_32`: Archives integral `*size` types as 32-bit integers. Enabled by default.
+//! - `size_64`: Archives integral `*size` types as 64-bit integers. This is intended to be used
+//!   only for very large archives and may cause unnecessary data bloat.
+//! - `std`: Enables standard library support. Enabled by default.
 //! - `strict`: Guarantees that types will have the same representations across platforms and
-//!   compilations. This is already the case in practice, but this feature provides a guarantee. It
-//!   additionally provides C type compatibility.
-//! - `validation`: Enables validation support through `bytecheck`
+//!   compilations. This is already the case in practice, but this feature provides a guarantee
+//!   along with C type compatibility.
+//! - `validation`: Enables validation support through `bytecheck`.
+//!
+//! ## Crate support
+//!
+//! Some common crates need to be supported by rkyv before an official integration has been made.
+//! Support is provided by rkyv for these crates, but in the future crates should depend on rkyv and
+//! provide their own implementations. The crates that already have support provided by rkyv should
+//! work toward integrating the implementations into themselves.
+//!
+//! Crates supported by rkyv:
+//!
+//! - [`indexmap`](https://docs.rs/indexmap)
+//! - [`tinyvec`](https://docs.rs/tinyvec)
+//! - [`uuid`](https://docs.rs/uuid)
+//!
+//! Features:
+//!
+//! - `tinyvec_alloc`: Supports types behind the `alloc` feature in `tinyvec`.
+//! - `uuid_std`: Enables the `std` feature in `uuid`.
 //!
 //! ## Examples
 //!
 //! See [`Archive`] for examples of how to use rkyv.
 
+#![deny(broken_intra_doc_links)]
+#![deny(missing_docs)]
+#![deny(missing_crate_level_docs)]
 #![cfg_attr(not(feature = "std"), no_std)]
-#![cfg_attr(feature = "specialization", feature(min_specialization))]
-#![cfg_attr(feature = "specialization", feature(rustc_attrs))]
+#![cfg_attr(feature = "copy", feature(auto_traits))]
+#![cfg_attr(feature = "copy", feature(min_specialization))]
+#![cfg_attr(feature = "copy", feature(negative_impls))]
+#![cfg_attr(feature = "copy", feature(rustc_attrs))]
+#![doc(html_favicon_url = r#"
+    data:image/svg+xml,%3Csvg xmlns='http://www.w3.org/2000/svg'
+    viewBox='0 0 26.458 26.458'%3E%3Cpath d='M0 0v26.458h26.458V0zm9.175 3.772l8.107 8.106
+    2.702-2.702 2.702 13.512-13.512-2.702 2.703-2.702-8.107-8.107z'/%3E%3C/svg%3E
+"#)]
+#![doc(html_logo_url = r#"
+    data:image/svg+xml,%3Csvg xmlns="http://www.w3.org/2000/svg" width="100" height="100"
+    viewBox="0 0 26.458 26.458"%3E%3Cpath d="M0 0v26.458h26.458V0zm9.175 3.772l8.107 8.106
+    2.702-2.702 2.702 13.512-13.512-2.702 2.703-2.702-8.107-8.107z"/%3E%3C/svg%3E
+"#)]
+
+#[cfg(feature = "alloc")]
+extern crate alloc;
+#[cfg(feature = "std")]
+extern crate std;
 
 #[doc(hidden)]
 #[macro_use]
 pub mod macros;
-pub mod core_impl;
+
+pub mod boxed;
+pub mod collections;
+#[cfg(feature = "copy")]
+pub mod copy;
 pub mod de;
-pub mod ser;
+// This is pretty unfortunate. CStr doesn't rely on the rest of std, but it's not in core.
+// If CStr ever gets moved into `core` then this module will no longer need cfg(feature = "std")
 #[cfg(feature = "std")]
-pub mod std_impl;
+pub mod ffi;
+mod impls;
+pub mod net;
+pub mod ops;
+pub mod option;
+pub mod rc;
+pub mod rel_ptr;
+pub mod ser;
+pub mod string;
+pub mod time;
 pub mod util;
 #[cfg(feature = "validation")]
 pub mod validation;
+pub mod vec;
+pub mod with;
 
-use core::{
-    fmt,
-    marker::{PhantomData, PhantomPinned},
-    mem::MaybeUninit,
-};
-
-pub use memoffset::{offset_of, offset_of_tuple};
+use core::alloc::Layout;
 use ptr_meta::Pointee;
 pub use rkyv_derive::{Archive, Deserialize, Serialize};
 pub use util::*;
 #[cfg(feature = "validation")]
-pub use validation::{check_archived_root, check_archived_value};
+#[cfg_attr(doc_cfg, doc(cfg(feature = "validation")))]
+pub use validation::{
+    check_archived_root_with_context, check_archived_value_with_context,
+    validators::{check_archived_root, check_archived_value},
+};
 
-/// Contains the error type for traits with methods that can fail
+/// A type that can produce an error.
+///
+/// This trait is always implemented by serializers and deserializers. Its purpose is to provide an
+/// error type without restricting what other capabilities the type must provide.
+///
+/// When writing implementations for [`Serialize`] and [`Deserialize`], it's best practice to bound
+/// the serializer or deserializer by `Fallible` and then require that the serialized types support
+/// it (i.e. `S: Fallible, MyType: Serialize<S>`).
 pub trait Fallible {
-    /// The error produced by any failing methods
+    /// The error produced by any failing methods.
     type Error: 'static;
 }
 
-/// An error that can never be produced
-#[derive(Debug)]
-pub enum Unreachable {}
-
-impl fmt::Display for Unreachable {
-    fn fmt(&self, _: &mut fmt::Formatter<'_>) -> fmt::Result {
-        unsafe { core::hint::unreachable_unchecked() }
-    }
-}
-
-#[cfg(feature = "std")]
-impl std::error::Error for Unreachable {}
-
-/// A fallible type that cannot produce errors
+/// A fallible type that cannot produce errors.
+///
+/// This type can be used to serialize and deserialize types that cannot fail to serialize or
+/// deserialize.
 #[derive(Debug)]
 pub struct Infallible;
 
 impl Fallible for Infallible {
-    type Error = Unreachable;
+    type Error = core::convert::Infallible;
+}
+
+impl Default for Infallible {
+    fn default() -> Self {
+        Infallible
+    }
 }
 
 /// A type that can be used without deserializing.
 ///
+/// `Archive` is one of three basic traits used to work with zero-copy data and controls the layout
+/// of the data in its archived zero-copy representation. The [`Serialize`] trait helps transform
+/// types into that representation, and the [`Deserialize`] trait helps transform types back out.
+///
+/// Types that implement `Archive` must have a well-defined archived size. Unsized types can be
+/// supported using the [`ArchiveUnsized`] trait, along with [`SerializeUnsized`] and
+/// [`DeserializeUnsized`].
+///
 /// Archiving is done depth-first, writing any data owned by a type before writing the data for the
 /// type itself. The type must be able to create the archived type from only its own data and its
 /// resolver.
 ///
-/// ## Examples
+/// Archived data is always treated as if it is tree-shaped, with the root owning its direct
+/// descendents and so on. Data that is not tree-shaped can be supported using special serializer
+/// and deserializer bounds (see [`ArchivedRc`](crate::rc::ArchivedRc) for example). In a buffer of
+/// serialized data, objects are laid out in *reverse order*. This means that the root object is
+/// located near the end of the buffer and leaf objects are located near the beginning.
+///
+/// # Examples
 ///
 /// Most of the time, `#[derive(Archive)]` will create an acceptable implementation. You can use the
-/// `#[archive(...)]` attribute to control how the implementation is generated. See the
-/// [`Archive`](macro@Archive) derive macro for more details.
+/// `#[archive(...)]` and `#[archive_attr(...)]` attributes to control how the implementation is
+/// generated. See the [`Archive`](macro@Archive) derive macro for more details.
 ///
 /// ```
 /// use rkyv::{
 ///     archived_root,
-///     de::deserializers::AllocDeserializer,
-///     ser::{Serializer, serializers::AlignedSerializer},
-///     AlignedVec,
-///     Archive,
-///     Archived,
-///     Deserialize,
-///     Serialize,
+///     ser::{Serializer, serializers::AllocSerializer},
+///     AlignedVec, Archive, Deserialize, Infallible, Serialize,
 /// };
 ///
 /// #[derive(Archive, Serialize, Deserialize, Debug, PartialEq)]
@@ -155,16 +233,16 @@ impl Fallible for Infallible {
 ///     option: Some(vec![1, 2, 3, 4]),
 /// };
 ///
-/// let mut serializer = AlignedSerializer::new(AlignedVec::new());
-/// serializer.serialize_value(&value).expect("failed to archive test");
-/// let buf = serializer.into_inner();
+/// let mut serializer = AllocSerializer::<4096>::default();
+/// serializer.serialize_value(&value).unwrap();
+/// let buf = serializer.into_serializer().into_inner();
 ///
 /// let archived = unsafe { archived_root::<Test>(buf.as_slice()) };
 /// assert_eq!(archived.int, value.int);
 /// assert_eq!(archived.string, value.string);
 /// assert_eq!(archived.option, value.option);
 ///
-/// let deserialized = archived.deserialize(&mut AllocDeserializer).unwrap();
+/// let deserialized = archived.deserialize(&mut Infallible).unwrap();
 /// assert_eq!(value, deserialized);
 /// ```
 ///
@@ -173,16 +251,16 @@ impl Fallible for Infallible {
 /// handle.
 ///
 /// In this example, we add our own wrapper that serializes a `&'static str` as if it's owned.
-/// Normally you can lean on the archived version of `String` to do most of the work, but this
-/// example does everything to demonstrate how to implement `Archive` for your own types.
+/// Normally you can lean on the archived version of `String` to do most of the work, or use the
+/// [`Inline`](crate::with::Inline) to do exactly this. This example does everything to demonstrate
+/// how to implement `Archive` for your own types.
 ///
 /// ```
-/// use core::{mem::MaybeUninit, slice, str};
+/// use core::{slice, str};
 /// use rkyv::{
 ///     archived_root,
-///     offset_of,
-///     project_struct,
 ///     ser::{Serializer, serializers::AlignedSerializer},
+///     out_field,
 ///     AlignedVec,
 ///     Archive,
 ///     Archived,
@@ -231,23 +309,22 @@ impl Fallible for Infallible {
 ///
 ///     // The resolve function consumes the resolver and produces the archived
 ///     // value at the given position.
-///     fn resolve(
+///     unsafe fn resolve(
 ///         &self,
 ///         pos: usize,
 ///         resolver: Self::Resolver,
-///         out: &mut MaybeUninit<Self::Archived>
+///         out: *mut Self::Archived,
 ///     ) {
 ///         // We have to be careful to add the offset of the ptr field,
 ///         // otherwise we'll be using the position of the ArchivedOwnedStr
 ///         // instead of the position of the relative pointer.
-///         unsafe {
-///             self.inner.resolve_unsized(
-///                 pos + offset_of!(Self::Archived, ptr),
-///                 resolver.pos,
-///                 resolver.metadata_resolver,
-///                 project_struct!(out: Self::Archived => ptr),
-///             );
-///         }
+///         let (fp, fo) = out_field!(out.ptr);
+///         self.inner.resolve_unsized(
+///             pos + fp,
+///             resolver.pos,
+///             resolver.metadata_resolver,
+///             fo,
+///         );
 ///     }
 /// }
 ///
@@ -280,25 +357,38 @@ impl Fallible for Infallible {
 /// assert_eq!(archived.as_str(), STR_VAL);
 /// ```
 pub trait Archive {
-    /// The archived version of this type.
+    /// The archived representation of this type.
+    ///
+    /// In this form, the data can be used with zero-copy deserialization.
     type Archived;
 
-    /// The resolver for this type. It must contain all the information needed to make the archived
-    /// type from the normal type.
+    /// The resolver for this type. It must contain all the additional information from serializing
+    /// needed to make the archived type from the normal type.
     type Resolver;
 
     /// Creates the archived version of this value at the given position and writes it to the given
     /// output.
     ///
-    /// The output should be initialized field-by-field rather than by writing a whole struct. This
-    /// is because performing a typed copy will set all of the padding bytes to uninitialized, but
-    /// they must remain whatever value they currently have. This is so that uninitialized memory
-    /// doesn't get leaked to the final archive.
-    fn resolve(&self, pos: usize, resolver: Self::Resolver, out: &mut MaybeUninit<Self::Archived>);
+    /// The output should be initialized field-by-field rather than by writing a whole struct.
+    /// Performing a typed copy will mark all of the padding bytes as uninitialized, but they must
+    /// remain set to the value they currently have. This prevents leaking uninitialized memory to
+    /// the final archive.
+    ///
+    /// # Safety
+    ///
+    /// - `pos` must be the position of `out` within the archive
+    /// - `resolver` must be the result of serializing this object
+    unsafe fn resolve(&self, pos: usize, resolver: Self::Resolver, out: *mut Self::Archived);
 }
 
 /// Converts a type to its archived form.
 ///
+/// Objects perform any supportive serialization during [`serialize`](Serialize::serialize). For
+/// types that reference nonlocal (pointed-to) data, this is when that data must be serialized to
+/// the output. These types will need to bound `S` to implement [`Serializer`](ser::Serializer) and
+/// any other required traits (e.g. [`SharedSerializeRegistry`](ser::SharedSerializeRegistry)). They
+/// should then serialize their dependencies during `serialize`.
+///
 /// See [`Archive`] for examples of implementing `Serialize`.
 pub trait Serialize<S: Fallible + ?Sized>: Archive {
     /// Writes the dependencies for the object and returns a resolver that can create the archived
@@ -308,42 +398,49 @@ pub trait Serialize<S: Fallible + ?Sized>: Archive {
 
 /// Converts a type back from its archived form.
 ///
+/// Some types may require specific deserializer capabilities, such as `Rc` and `Arc`. In these
+/// cases, the deserializer type `D` should be bound so that it implements traits that provide those
+/// capabilities (e.g. [`SharedDeserializeRegistry`](de::SharedDeserializeRegistry)).
+///
 /// This can be derived with [`Deserialize`](macro@Deserialize).
-pub trait Deserialize<T: Archive<Archived = Self>, D: Fallible + ?Sized> {
+pub trait Deserialize<T, D: Fallible + ?Sized> {
     /// Deserializes using the given deserializer
     fn deserialize(&self, deserializer: &mut D) -> Result<T, D::Error>;
 }
 
 /// A counterpart of [`Archive`] that's suitable for unsized types.
 ///
-/// Instead of archiving its value directly, `ArchiveUnsized` archives a [`RelPtr`] to its archived
-/// type. As a consequence, its resolver must be `usize`.
+/// Unlike `Archive`, types that implement `ArchiveUnsized` must be serialized separately from their
+/// owning object. For example, whereas an `i32` might be laid out as part of a larger struct, a
+/// `Box<i32>` would serialize the `i32` somewhere in the archive and the `Box` would point to it as
+/// part of the larger struct. Because of this, the equivalent [`Resolver`](Archive::Resolver) type
+/// for `ArchiveUnsized` is always a `usize` representing the position of the serialized value.
 ///
-/// `ArchiveUnsized` is automatically implemented for all types that implement [`Archive`].
+/// `ArchiveUnsized` is automatically implemented for all types that implement [`Archive`]. Nothing
+/// special needs to be done to use them with types like `Box`, `Rc`, and `Arc`. It is also already
+/// implemented for slices and string slices, and the `rkyv_dyn` crate can be used to archive trait
+/// objects. Other unsized types must manually implement `ArchiveUnsized`.
 ///
-/// `ArchiveUnsized` is already implemented for slices and string slices, and the `rkyv_dyn` crate
-/// can be used to archive trait objects. Other unsized types must manually implement
-/// `ArchiveUnsized`.
+/// # Examples
 ///
-/// ## Examples
+/// This example shows how to manually implement `ArchiveUnsized` for an unsized type. Special care
+/// must be taken to ensure that the types are laid out correctly.
 ///
 /// ```
-/// use core::{
-///     mem::{transmute, MaybeUninit},
-///     ops::{Deref, DerefMut},
-/// };
+/// use core::{mem::transmute, ops::{Deref, DerefMut}};
 /// use ptr_meta::Pointee;
 /// use rkyv::{
+///     from_archived,
+///     to_archived,
 ///     archived_unsized_value,
-///     offset_of,
 ///     ser::{serializers::AlignedSerializer, Serializer},
 ///     AlignedVec,
 ///     Archive,
 ///     Archived,
 ///     ArchivedMetadata,
-///     ArchivedUsize,
 ///     ArchivePointee,
 ///     ArchiveUnsized,
+///     FixedUsize,
 ///     RelPtr,
 ///     Serialize,
 ///     SerializeUnsized,
@@ -362,7 +459,7 @@ pub trait Deserialize<T: Archive<Archived = Self>, D: Fallible + ?Sized> {
 /// // For blocks with trailing slices, we need to store the length of the slice
 /// // in the metadata.
 /// pub struct BlockSliceMetadata {
-///     len: ArchivedUsize,
+///     len: Archived<usize>,
 /// }
 ///
 /// // ArchivePointee is automatically derived for sized types because pointers
@@ -379,7 +476,7 @@ pub trait Deserialize<T: Archive<Archived = Self>, D: Fallible + ?Sized> {
 ///     fn pointer_metadata(
 ///         archived: &Self::ArchivedMetadata
 ///     ) -> <Self as Pointee>::Metadata {
-///         archived.len as usize
+///         from_archived!(archived.len) as usize
 ///     }
 /// }
 ///
@@ -397,15 +494,15 @@ pub trait Deserialize<T: Archive<Archived = Self>, D: Fallible + ?Sized> {
 ///     // Here's where we make the metadata for our pointer.
 ///     // This also gets the position and resolver for the metadata, but we
 ///     // don't need it in this case.
-///     fn resolve_metadata(
+///     unsafe fn resolve_metadata(
 ///         &self,
 ///         _: usize,
 ///         _: Self::MetadataResolver,
-///         out: &mut MaybeUninit<ArchivedMetadata<Self>>,
+///         out: *mut ArchivedMetadata<Self>,
 ///     ) {
 ///         unsafe {
-///             out.as_mut_ptr().write(BlockSliceMetadata {
-///                 len: self.tail.len() as ArchivedUsize,
+///             out.write(BlockSliceMetadata {
+///                 len: to_archived!(self.tail.len() as FixedUsize),
 ///             });
 ///         }
 ///     }
@@ -483,34 +580,56 @@ pub trait Deserialize<T: Archive<Archived = Self>, D: Fallible + ?Sized> {
 /// ```
 pub trait ArchiveUnsized: Pointee {
     /// The archived counterpart of this type. Unlike `Archive`, it may be unsized.
+    ///
+    /// This type must implement [`ArchivePointee`], a trait that helps make valid pointers using
+    /// archived pointer metadata.
     type Archived: ArchivePointee + ?Sized;
 
     /// The resolver for the metadata of this type.
+    ///
+    /// Because the pointer metadata must be archived with the relative pointer and not with the
+    /// structure itself, its resolver must be passed back to the structure holding the pointer.
     type MetadataResolver;
 
     /// Creates the archived version of the metadata for this value at the given position and writes
     /// it to the given output.
     ///
-    /// The output should be initialized field-by-field rather than by writing a whole struct. This
-    /// is because performing a typed copy will set all of the padding bytes to uninitialized, but
-    /// they must remain whatever value they currently have. This is so that uninitialized memory
-    /// doesn't get leaked to the final archive.
-    fn resolve_metadata(
+    /// The output should be initialized field-by-field rather than by writing a whole struct.
+    /// Performing a typed copy will mark all of the padding bytes as uninitialized, but they must
+    /// remain set to the value they currently have. This prevents leaking uninitialized memory to
+    /// the final archive.
+    ///
+    /// # Safety
+    ///
+    /// - `pos` must be the position of `out` within the archive
+    /// - `resolver` must be the result of serializing this object's metadata
+    unsafe fn resolve_metadata(
         &self,
         pos: usize,
         resolver: Self::MetadataResolver,
-        out: &mut MaybeUninit<ArchivedMetadata<Self>>,
+        out: *mut ArchivedMetadata<Self>,
     );
 
     /// Resolves a relative pointer to this value with the given `from` and `to` and writes it to
     /// the given output.
+    ///
+    /// The output should be initialized field-by-field rather than by writing a whole struct.
+    /// Performing a typed copy will mark all of the padding bytes as uninitialized, but they must
+    /// remain set to the value they currently have. This prevents leaking uninitialized memory to
+    /// the final archive.
+    ///
+    /// # Safety
+    ///
+    /// - `from` must be the position of `out` within the archive
+    /// - `to` must be the position of some `Self::Archived` within the archive
+    /// - `resolver` must be the result of serializing this object
     #[inline]
-    fn resolve_unsized(
+    unsafe fn resolve_unsized(
         &self,
         from: usize,
         to: usize,
         resolver: Self::MetadataResolver,
-        out: &mut MaybeUninit<RelPtr<Self::Archived>>,
+        out: *mut RelPtr<Self::Archived>,
     ) {
         RelPtr::resolve_emplace(from, to, self, resolver, out);
     }
@@ -540,255 +659,59 @@ pub trait SerializeUnsized<S: Fallible + ?Sized>: ArchiveUnsized {
 }
 
 /// A counterpart of [`Deserialize`] that's suitable for unsized types.
-///
-/// Most types that implement `DeserializeUnsized` will need a [`Deserializer`](de::Deserializer)
-/// bound so that they can allocate memory.
-pub trait DeserializeUnsized<T: ArchiveUnsized<Archived = Self> + ?Sized, D: Fallible + ?Sized>:
-    ArchivePointee
-{
+pub trait DeserializeUnsized<T: Pointee + ?Sized, D: Fallible + ?Sized>: ArchivePointee {
     /// Deserializes a reference to the given value.
     ///
     /// # Safety
     ///
-    /// The caller must guarantee that the memory returned is properly deallocated.
-    unsafe fn deserialize_unsized(&self, deserializer: &mut D) -> Result<*mut (), D::Error>;
+    /// `out` must point to memory with the layout returned by `deserialized_layout`.
+    unsafe fn deserialize_unsized(
+        &self,
+        deserializer: &mut D,
+        alloc: impl FnMut(Layout) -> *mut u8,
+    ) -> Result<*mut (), D::Error>;
 
     /// Deserializes the metadata for the given type.
     fn deserialize_metadata(&self, deserializer: &mut D) -> Result<T::Metadata, D::Error>;
 }
 
-/// An [`Archive`] type that is a bitwise copy of itself and without additional processing.
+#[cfg(not(any(feature = "size_16", feature = "size_32", feature = "size_64")))]
+core::compile_error!("\"size_16\", \"size_32\", or \"size_64\" feature must be eanbled for rkyv.");
+
+/// The native type that `usize` is converted to for archiving.
 ///
-/// Types that implement `ArchiveCopy` are not guaranteed to have a [`Serialize`] implementation
-/// called on them to archive their value.
+/// This will be `u16`, `u32`, or `u64` when the `size_16`, `size_32`, or `size_64` features are
+/// enabled, respectively.
+pub type FixedUsize = pick_size_type!(u16, u32, u64);
+/// The native type that `isize` is converted to for archiving.
 ///
-/// You can derive an implementation of `ArchiveCopy` by adding `#[archive(copy)]` to the struct or
-/// enum. Types that implement `ArchiveCopy` must also implement [`Copy`](core::marker::Copy).
+/// This will be `i16`, `i32`, or `i64` when the `size_16`, `size_32`, or `size_64` features are
+/// enabled, respectively.
+pub type FixedIsize = pick_size_type!(i16, i32, i64);
+
+/// The default raw relative pointer.
 ///
-/// `ArchiveCopy` must be manually implemented even if a type implements [`Archive`] and
-/// [`Copy`](core::marker::Copy) because some types may transform their data when writing to an
-/// archive.
+/// This will use an archived [`FixedIsize`] to hold the offset.
+pub type RawRelPtr = rel_ptr::RawRelPtr<Archived<isize>>;
+/// The default relative pointer.
 ///
-/// ## Examples
-/// ```
-/// use rkyv::{
-///     archived_root,
-///     ser::{Serializer, serializers::AlignedSerializer},
-///     AlignedVec,
-///     Archive,
-///     Serialize,
-/// };
-///
-/// #[derive(Archive, Serialize, Clone, Copy, Debug, PartialEq)]
-/// #[archive(copy)]
-/// struct Vector4<T>(T, T, T, T);
-///
-/// let mut serializer = AlignedSerializer::new(AlignedVec::new());
-/// let value = Vector4(1f32, 2f32, 3f32, 4f32);
-/// serializer.serialize_value(&value).expect("failed to archive Vector4");
-/// let buf = serializer.into_inner();
-/// let archived_value = unsafe { archived_root::<Vector4<f32>>(buf.as_ref()) };
-/// assert_eq!(&value, archived_value);
-/// ```
-#[cfg_attr(feature = "specialization", rustc_unsafe_specialization_marker)]
-pub unsafe trait ArchiveCopy: Archive<Archived = Self> + Copy {}
-
-/// The type used for sizes in archived types.
-#[cfg(not(feature = "size_64"))]
-pub type ArchivedUsize = u32;
-
-/// The type used for offsets in relative pointers.
-#[cfg(not(feature = "size_64"))]
-pub type ArchivedIsize = i32;
-
-/// The type used for sizes in archived types.
-#[cfg(feature = "size_64")]
-pub type ArchivedUsize = u64;
-
-/// The type used for offsets in relative pointers.
-#[cfg(feature = "size_64")]
-pub type ArchivedIsize = i64;
-
-/// An untyped pointer which resolves relative to its position in memory.
-#[derive(Debug)]
-#[repr(transparent)]
-pub struct RawRelPtr {
-    offset: ArchivedIsize,
-    _phantom: PhantomPinned,
-}
-
-impl RawRelPtr {
-    /// Emplaces a new relative pointer between the given positions and stores it in the given
-    /// output.
-    #[inline]
-    pub fn emplace(from: usize, to: usize, out: &mut MaybeUninit<Self>) {
-        let offset = (to as isize - from as isize) as ArchivedIsize;
-        unsafe {
-            project_struct!(out: Self => offset: ArchivedIsize)
-                .as_mut_ptr()
-                .write(offset);
-        }
-    }
-
-    /// Creates a new relative pointer that has an offset of 0.
-    #[inline]
-    pub fn null() -> Self {
-        Self {
-            offset: 0,
-            _phantom: PhantomPinned,
-        }
-    }
-
-    /// Checks whether the relative pointer is null.
-    #[inline]
-    pub fn is_null(&self) -> bool {
-        self.offset == 0
-    }
-
-    /// Gets the base pointer for the relative pointer.
-    #[inline]
-    pub fn base(&self) -> *const u8 {
-        (self as *const Self).cast::<u8>()
-    }
-
-    /// Gets the offset of the relative pointer.
-    #[inline]
-    pub fn offset(&self) -> isize {
-        self.offset as isize
-    }
-
-    /// Calculates the memory address being pointed to by this relative pointer.
-    #[inline]
-    pub fn as_ptr(&self) -> *const () {
-        unsafe {
-            (self as *const Self)
-                .cast::<u8>()
-                .offset(self.offset as isize)
-                .cast()
-        }
-    }
-
-    /// Returns an unsafe mutable pointer to the memory address being pointed to
-    /// by this relative pointer.
-    #[inline]
-    pub fn as_mut_ptr(&mut self) -> *mut () {
-        unsafe {
-            (self as *mut Self)
-                .cast::<u8>()
-                .offset(self.offset as isize)
-                .cast()
-        }
-    }
-}
-
-/// A pointer which resolves to relative to its position in memory.
-///
-/// See [`Archive`] for an example of creating one.
-#[cfg_attr(feature = "strict", repr(C))]
-pub struct RelPtr<T: ArchivePointee + ?Sized> {
-    raw_ptr: RawRelPtr,
-    metadata: T::ArchivedMetadata,
-    _phantom: PhantomData<T>,
-}
-
-impl<T: ArchivePointee + ?Sized> RelPtr<T> {
-    /// Creates a new relative pointer from the given raw pointer and metadata.
-    ///
-    /// # Safety
-    ///
-    /// The caller must guarantee that:
-    /// - `raw_ptr` is a valid relative pointer in its final position
-    /// - `raw_ptr` points to a valid value
-    /// - `metadata` is valid metadata for the pointed value.
-    #[inline]
-    pub fn new(raw_ptr: RawRelPtr, metadata: T::ArchivedMetadata) -> Self {
-        Self {
-            raw_ptr,
-            metadata,
-            _phantom: PhantomData,
-        }
-    }
-
-    /// Creates a relative pointer from one position to another.
-    ///
-    /// # Safety
-    ///
-    /// The caller must guarantee that `from` is the position of the relative pointer and `to` is
-    /// the position of some valid memory.
-    #[inline]
-    pub fn resolve_emplace<U: ArchiveUnsized<Archived = T> + ?Sized>(
-        from: usize,
-        to: usize,
-        value: &U,
-        metadata_resolver: U::MetadataResolver,
-        out: &mut MaybeUninit<Self>,
-    ) {
-        RawRelPtr::emplace(
-            from + offset_of!(Self, raw_ptr),
-            to,
-            project_struct!(out: Self => raw_ptr),
-        );
-        value.resolve_metadata(
-            from + offset_of!(Self, metadata),
-            metadata_resolver,
-            project_struct!(out: Self => metadata),
-        );
-    }
-
-    /// Gets the base pointer for the relative pointer.
-    #[inline]
-    pub fn base(&self) -> *const u8 {
-        self.raw_ptr.base()
-    }
-
-    /// Gets the offset of the relative pointer.
-    #[inline]
-    pub fn offset(&self) -> isize {
-        self.raw_ptr.offset()
-    }
-
-    /// Gets the metadata of the relative pointer.
-    #[inline]
-    pub fn metadata(&self) -> &T::ArchivedMetadata {
-        &self.metadata
-    }
-
-    /// Calculates the memory address being pointed to by this relative pointer.
-    #[inline]
-    pub fn as_ptr(&self) -> *const T {
-        ptr_meta::from_raw_parts(self.raw_ptr.as_ptr(), T::pointer_metadata(&self.metadata))
-    }
-
-    /// Returns an unsafe mutable pointer to the memory address being pointed to by this relative
-    /// pointer.
-    #[inline]
-    pub fn as_mut_ptr(&mut self) -> *mut T {
-        ptr_meta::from_raw_parts_mut(
-            self.raw_ptr.as_mut_ptr(),
-            T::pointer_metadata(&self.metadata),
-        )
-    }
-}
-
-impl<T: ArchivePointee + ?Sized> fmt::Debug for RelPtr<T>
-where
-    T::ArchivedMetadata: fmt::Debug,
-{
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        f.debug_struct("RelPtr")
-            .field("raw_ptr", &self.raw_ptr)
-            .field("metadata", &self.metadata)
-            .field("_phantom", &self._phantom)
-            .finish()
-    }
-}
+/// This will use an archived [`FixedIsize`] to hold the offset.
+pub type RelPtr<T> = rel_ptr::RelPtr<T, Archived<isize>>;
 
 /// Alias for the archived version of some [`Archive`] type.
+///
+/// This can be useful for reducing the lengths of type definitions.
 pub type Archived<T> = <T as Archive>::Archived;
 /// Alias for the resolver for some [`Archive`] type.
+///
+/// This can be useful for reducing the lengths of type definitions.
 pub type Resolver<T> = <T as Archive>::Resolver;
 /// Alias for the archived metadata for some [`ArchiveUnsized`] type.
+///
+/// This can be useful for reducing the lengths of type definitions.
 pub type ArchivedMetadata<T> =
     <<T as ArchiveUnsized>::Archived as ArchivePointee>::ArchivedMetadata;
 /// Alias for the metadata resolver for some [`ArchiveUnsized`] type.
+///
+/// This can be useful for reducing the lengths of type definitions.
 pub type MetadataResolver<T> = <T as ArchiveUnsized>::MetadataResolver;
