@@ -9,8 +9,9 @@
 //! Alignment helpers ensure that byte buffers are properly aligned when accessing and deserializing
 //! data.
 
-#[cfg(feature = "std")]
-mod std;
+#[cfg(feature = "alloc")]
+mod aligned_vec;
+mod scratch_vec;
 
 use crate::{Archive, ArchiveUnsized, RelPtr};
 use core::{
@@ -20,8 +21,10 @@ use core::{
 };
 
 #[doc(inline)]
-#[cfg(feature = "std")]
-pub use self::std::*;
+#[cfg(feature = "alloc")]
+pub use self::aligned_vec::*;
+#[doc(inline)]
+pub use self::scratch_vec::*;
 
 #[cfg(debug_assertions)]
 #[inline]
@@ -44,7 +47,7 @@ fn check_alignment<T>(ptr: *const u8) {
 ///
 /// # Safety
 ///
-/// The caller must guarantee that a value is archived at the given position in the byte slice.
+/// A `T::Archived` must be archived at the given position in the byte slice.
 #[inline]
 pub unsafe fn archived_value<T: Archive + ?Sized>(bytes: &[u8], pos: usize) -> &T::Archived {
     #[cfg(debug_assertions)]
@@ -60,7 +63,7 @@ pub unsafe fn archived_value<T: Archive + ?Sized>(bytes: &[u8], pos: usize) -> &
 ///
 /// # Safety
 ///
-/// The caller must guarantee that a value is archived at the given position in the byte slice.
+/// A `T::Archived` must be archived at the given position in the byte slice.
 #[inline]
 pub unsafe fn archived_value_mut<T: Archive + ?Sized>(
     bytes: Pin<&mut [u8]>,
@@ -80,7 +83,7 @@ pub unsafe fn archived_value_mut<T: Archive + ?Sized>(
 ///
 /// # Safety
 ///
-/// The caller must guarantee that a reference is archived at the given position in the byte slice.
+/// A `RelPtr<T::Archived>` must be archived at the given position in the byte slice.
 #[inline]
 pub unsafe fn archived_unsized_value<T: ArchiveUnsized + ?Sized>(
     bytes: &[u8],
@@ -101,7 +104,7 @@ pub unsafe fn archived_unsized_value<T: ArchiveUnsized + ?Sized>(
 ///
 /// # Safety
 ///
-/// The caller must guarantee that a reference is archived at the given position in the byte slice.
+/// A `RelPtr<T::Archived>` must be archived at the given position in the byte slice.
 #[inline]
 pub unsafe fn archived_unsized_value_mut<T: ArchiveUnsized + ?Sized>(
     bytes: Pin<&mut [u8]>,
@@ -122,13 +125,13 @@ pub unsafe fn archived_unsized_value_mut<T: ArchiveUnsized + ?Sized>(
 ///
 /// This is a wrapper for [`archived_value`](crate::archived_value) that calculates the correct
 /// position of the root using the length of the byte slice. If your byte slice is not guaranteed to
-/// end immediately after the root object, you may need to store the position of the root object or
-/// use [`serialize_front`](crate::ser::SeekSerializer::serialize_front) instead.
+/// end immediately after the root object, you may need to store the position of the root object
+/// returned from [`serialize_value`](crate::ser::Serializer::serialize_value).
 ///
 /// # Safety
 ///
-/// The caller must guarantee that the byte slice represents an archived object and that the root
-/// object is stored at the end of the byte slice.
+/// - The byte slice must represent an archived object
+/// - The root of the object must be stored at the end of the slice (this is the default behavior)
 #[inline]
 pub unsafe fn archived_root<T: Archive + ?Sized>(bytes: &[u8]) -> &T::Archived {
     archived_value::<T>(bytes, bytes.len() - mem::size_of::<T::Archived>())
@@ -139,12 +142,12 @@ pub unsafe fn archived_root<T: Archive + ?Sized>(bytes: &[u8]) -> &T::Archived {
 /// This is a wrapper for [`archived_value_mut`](crate::archived_value_mut) that calculates the
 /// correct position of the root using the length of the byte slice. If your byte slice is not
 /// guaranteed to end immediately after the root object, you may need to store the position of the
-/// root object or use [`serialize_front`](crate::ser::SeekSerializer::serialize_front) instead.
+/// root object returned from [`serialize_value`](crate::ser::Serializer::serialize_value).
 ///
 /// # Safety
 ///
-/// The caller must guarantee that the byte slice represents an archived object and that the root
-/// object is stored at the end of the byte slice.
+/// - The byte slice must represent an archived object
+/// - The root of the object must be stored at the end of the slice (this is the default behavior)
 #[inline]
 pub unsafe fn archived_root_mut<T: Archive + ?Sized>(
     bytes: Pin<&mut [u8]>,
@@ -159,12 +162,13 @@ pub unsafe fn archived_root_mut<T: Archive + ?Sized>(
 /// This is a wrapper for [`archived_unsized_value`](crate::archived_unsized_value) that calculates
 /// the correct position of the root using the length of the byte slice. If your byte slice is not
 /// guaranteed to end immediately after the root object, you may need to store the position of the
-/// root object or use [`serialize_front`](crate::ser::SeekSerializer::serialize_front) instead.
+/// root object returned from
+/// [`serialize_unsized_value`](crate::ser::Serializer::serialize_unsized_value).
 ///
 /// # Safety
 ///
-/// The caller must guarantee that the byte slice represents an archived object and that the root
-/// object is stored at the end of the byte slice.
+/// - The byte slice must represent an archived object
+/// - The root of the object must be stored at the end of the slice (this is the default behavior)
 #[inline]
 pub unsafe fn archived_unsized_root<T: ArchiveUnsized + ?Sized>(bytes: &[u8]) -> &T::Archived {
     archived_unsized_value::<T>(bytes, bytes.len() - mem::size_of::<RelPtr<T::Archived>>())
@@ -176,13 +180,13 @@ pub unsafe fn archived_unsized_root<T: ArchiveUnsized + ?Sized>(bytes: &[u8]) ->
 /// This is a wrapper for [`archived_unsized_value_mut`](crate::archived_unsized_value_mut) that
 /// calculates the correct position of the root using the length of the byte slice. If your byte
 /// slice is not guaranteed to end immediately after the root object, you may need to store the
-/// position of the root object or use
-/// [`serialize_front`](crate::ser::SeekSerializer::serialize_front) instead.
+/// position of the root object returned from
+/// [`serialize_unsized_value`](crate::ser::Serializer::serialize_unsized_value).
 ///
 /// # Safety
 ///
-/// The caller must guarantee that the byte slice represents an archived object and that the root
-/// object is stored at the end of the byte slice.
+/// - The byte slice must represent an archived object
+/// - The root of the object must be stored at the end of the slice (this is the default behavior)
 #[inline]
 pub unsafe fn archived_unsized_root_mut<T: ArchiveUnsized + ?Sized>(
     bytes: Pin<&mut [u8]>,
@@ -191,46 +195,53 @@ pub unsafe fn archived_unsized_root_mut<T: ArchiveUnsized + ?Sized>(
     archived_unsized_value_mut::<T>(bytes, pos)
 }
 
-/// Wraps a type and aligns it to 16 bytes.
+/// A buffer of bytes aligned to 16 bytes.
+///
+/// # Examples
 ///
-/// ## Examples
 /// ```
 /// use core::mem;
-/// use rkyv::Aligned;
+/// use rkyv::AlignedBytes;
 ///
 /// assert_eq!(mem::align_of::<u8>(), 1);
-/// assert_eq!(mem::align_of::<Aligned<u8>>(), 16);
+/// assert_eq!(mem::align_of::<AlignedBytes<256>>(), 16);
 /// ```
 #[derive(Clone, Copy)]
-#[repr(align(16))]
-pub struct Aligned<T>(pub T);
+#[repr(C, align(16))]
+pub struct AlignedBytes<const N: usize>(pub [u8; N]);
 
-impl<T: Deref> Deref for Aligned<T> {
-    type Target = T::Target;
+impl<const N: usize> Default for AlignedBytes<N> {
+    fn default() -> Self {
+        Self([0; N])
+    }
+}
+
+impl<const N: usize> Deref for AlignedBytes<N> {
+    type Target = [u8; N];
 
     #[inline]
     fn deref(&self) -> &Self::Target {
-        &*self.0
+        &self.0
     }
 }
 
-impl<T: DerefMut> DerefMut for Aligned<T> {
+impl<const N: usize> DerefMut for AlignedBytes<N> {
     #[inline]
     fn deref_mut(&mut self) -> &mut Self::Target {
-        &mut *self.0
+        &mut self.0
     }
 }
 
-impl<T: AsRef<[U]>, U> AsRef<[U]> for Aligned<T> {
+impl<const N: usize> AsRef<[u8]> for AlignedBytes<N> {
     #[inline]
-    fn as_ref(&self) -> &[U] {
+    fn as_ref(&self) -> &[u8] {
         self.0.as_ref()
     }
 }
 
-impl<T: AsMut<[U]>, U> AsMut<[U]> for Aligned<T> {
+impl<const N: usize> AsMut<[u8]> for AlignedBytes<N> {
     #[inline]
-    fn as_mut(&mut self) -> &mut [U] {
+    fn as_mut(&mut self) -> &mut [u8] {
         self.0.as_mut()
     }
 }
